'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var caniuse = require('caniuse-lite');
var postcss = _interopDefault(require('postcss'));
var browserslist = _interopDefault(require('browserslist'));

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

const NO = 'No';
const APPLE_SYSTEM = 'AppleSystem';
const BLINK_MAC_SYSTEM_FONT = 'BlinkMacSystemFont';

function computePolyfillFlagFromStatus(statusString) {
	const flags = [];
	const statusArr = statusString.split(' ');
	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = statusArr[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			const status = _step.value;

			if (status === 'y') {
				return [];
			} else if (status.includes('n')) {
				flags.push(NO);
			} else if (status.includes('#1')) {
				// #1: Supported as the -apple-system value (only on Mac)
				flags.push(APPLE_SYSTEM);
			} else if (status.includes('#2')) {
				// #2: Supported as the 'BlinkMacSystemFont' value (only on Mac)
				flags.push(BLINK_MAC_SYSTEM_FONT);
			}
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}

	return flags;
}

function generatePolyfillFlags(stats, supportedBrowsers) {
	const polyfillFlags = { [APPLE_SYSTEM]: false, [BLINK_MAC_SYSTEM_FONT]: false, [NO]: false };
	const visitedStatus = {};
	var _iteratorNormalCompletion2 = true;
	var _didIteratorError2 = false;
	var _iteratorError2 = undefined;

	try {
		for (var _iterator2 = supportedBrowsers[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
			const browserAndVersion = _step2.value;

			var _browserAndVersion$sp = browserAndVersion.split(' '),
			    _browserAndVersion$sp2 = _slicedToArray(_browserAndVersion$sp, 2);

			const browser = _browserAndVersion$sp2[0],
			      version = _browserAndVersion$sp2[1];

			const status = stats[browser][version];
			if (visitedStatus[status]) {
				continue;
			}
			visitedStatus[status] = true;
			const shouldPolyfillFlags = computePolyfillFlagFromStatus(status);
			shouldPolyfillFlags.forEach(flag => polyfillFlags[flag] = true);
		}
	} catch (err) {
		_didIteratorError2 = true;
		_iteratorError2 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion2 && _iterator2.return) {
				_iterator2.return();
			}
		} finally {
			if (_didIteratorError2) {
				throw _iteratorError2;
			}
		}
	}

	return polyfillFlags;
}

function getSystemUiFamily(supportedBrowsers) {
	var _caniuse$feature = caniuse.feature(caniuse.features['font-family-system-ui']);

	const stats = _caniuse$feature.stats;

	const polyfillFlags = generatePolyfillFlags(stats, supportedBrowsers);

	const result = ['system-ui'];
	const polyfill = [[APPLE_SYSTEM, ['-apple-system']], [BLINK_MAC_SYSTEM_FONT, ['BlinkMacSystemFont']], [NO, ['Segoe UI', // Windows >= Vista
	'Roboto', // Android >= 4
	'Noto Sans', // Plasma >= 5.5
	'Ubuntu', // Ubuntu >= 10.10
	'Cantarell', // GNOME >= 3
	'Helvetica Neue']]];
	return result.concat.apply(result, polyfill.filter(([flag]) => polyfillFlags[flag]).map(v => v[1]));
}

// tooling

// plugin
var index = postcss.plugin('postcss-system-ui-font', opts => {

	return (root, result) => {

		const family = Object(opts).family;
		let systemUiFamily = family;
		if (typeof family === 'string') {
			systemUiFamily = family.trim().split(/\s*,\s*/);
		} else if (!family) {
			var _Object = Object(opts);

			const browsers = _Object.browsers;

			// browsers supported by the configuration

			const supportedBrowsers = browserslist(browsers, {
				path: result.root.source && result.root.source.input && result.root.source.input.file,
				ignoreUnknownVersions: true
			});
			systemUiFamily = getSystemUiFamily(supportedBrowsers);
		}

		// system-ui and fallbacks match
		const whitespace = '[\\f\\n\\r\\x09\\x20]';
		const systemUiMatch = new RegExp(`(^|,|${whitespace}+)(?:system-ui${whitespace}*)(?:,${whitespace}*(?:${systemUiFamily.join('|')})${whitespace}*)?(,|$)`, 'i');

		// system-ui fallback replacement
		const systemUiReplace = `$1${systemUiFamily.join(', ')}$2`;

		// update font declarations to polyfill system-ui usage
		root.walkDecls(fontPropertyMatch, decl => {
			decl.value = decl.value.replace(systemUiMatch, systemUiReplace);
		});
	};
});

/* match the following properties:
 * - font
 * - font-family
 * - custom properties (see https://www.w3.org/TR/css-variables-1/#custom-property)
 */
const fontPropertyMatch = /(?:^(?:-|\\002d){2})|(?:^font(?:-family)?$)/i;

module.exports = index;
